#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "midiplay.h"


char AppName[]   = "Midnight Player";
char ClassName[] = "MainMIDIWindow";


char ProgDir[256];


HINSTANCE hInst;
HWND MainWindow;

long WindowX = 0, WindowY = 0;

bool PlayOnLoad = false;


extern bool OpenMIDI(HWND hwnd, char *file);


#pragma argsused
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR CmdLine, int CmdShow)
{
  InitCommonControls();

  time_t t;
  srand((unsigned)time(&t));

  DWORD EmptyData;

  // Reads configuration from registry
  HKEY SoftwareKey;
  if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE", 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &SoftwareKey, &EmptyData) == ERROR_SUCCESS)
  {
    HKEY SeraphKey;
    if (RegCreateKeyEx(SoftwareKey, "Seraph", 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &SeraphKey, &EmptyData) == ERROR_SUCCESS)
    {
      HKEY MidiPlayKey;
      if (RegCreateKeyEx(SeraphKey, "midiplay", 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &MidiPlayKey, &EmptyData) == ERROR_SUCCESS)
      {
        char Data[513];

        DWORD DontNeed = 512;
        DWORD Type = REG_SZ;

        if (CmdLine[0] == 0)
        {
          GetCurrentDirectory(255, ProgDir);
          RegSetValueEx(MidiPlayKey, "ProgDirectory", 0, REG_SZ, (BYTE *)ProgDir, 255);
        }
        else
          RegQueryValueEx(MidiPlayKey, "ProgDirectory", NULL, &Type, (BYTE *)ProgDir, &DontNeed);

        if (RegQueryValueEx(MidiPlayKey, "LastDirectory", NULL, &Type, (BYTE *)Data, &DontNeed) == ERROR_SUCCESS)
          SetCurrentDirectory(Data);
        if (RegQueryValueEx(MidiPlayKey, "WindowX", NULL, &Type, (BYTE *)Data, &DontNeed) == ERROR_SUCCESS)
          WindowX = atoi(Data);
        if (RegQueryValueEx(MidiPlayKey, "WindowY", NULL, &Type, (BYTE *)Data, &DontNeed) == ERROR_SUCCESS)
          WindowY = atoi(Data);
        if (RegQueryValueEx(MidiPlayKey, "Repeat", NULL, &Type, (BYTE *)Data, &DontNeed) ==  ERROR_SUCCESS)
          SetRepeat(atoi(Data) == 1 ? true : false);
        if (RegQueryValueEx(MidiPlayKey, "Shuffle", NULL, &Type, (BYTE *)Data, &DontNeed) == ERROR_SUCCESS)
          SetShuffle(atoi(Data) == 1 ? true : false);
        if (RegQueryValueEx(MidiPlayKey, "Intro", NULL, &Type, (BYTE *)Data, &DontNeed) == ERROR_SUCCESS)
          SetIntro(atoi(Data) == 1 ? true : false);
        if (RegQueryValueEx(MidiPlayKey, "PlayOnLoad", NULL, &Type, (BYTE *)Data, &DontNeed) == ERROR_SUCCESS)
          PlayOnLoad = (atoi(Data) == 1 ? true : false);

        RegCloseKey(MidiPlayKey);
      }
      RegCloseKey(SeraphKey);
    }
    RegCloseKey(SoftwareKey);
  }

  hInst = hInstance;

  WNDCLASS wc;
  memset(&wc, 0, sizeof(wc));
  wc.lpfnWndProc = (WNDPROC)MainWindowProc;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_MAIN));
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_MENU + 1);
  wc.lpszMenuName = MAKEINTRESOURCE(IDM_MAINMENU);
  wc.lpszClassName = ClassName;
  if (!RegisterClass(&wc))
  {
    MessageBox(NULL, "Error: Could not register window class", AppName, MB_OK);
    return 0;
  }

  DWORD wstyle = WS_CAPTION | WS_MINIMIZEBOX | WS_POPUP | WS_SYSMENU | WS_OVERLAPPED;

  RECT ClientRect = { 0, 0, 400, 100 };
  AdjustWindowRect(&ClientRect, wstyle, 1);

  MainWindow = CreateWindow(
    ClassName, AppName,
    wstyle,
    WindowX, WindowY, (ClientRect.right - ClientRect.left), (ClientRect.bottom - ClientRect.top),
    NULL, NULL, hInstance, NULL);
  if (MainWindow == NULL)
  {
    MessageBox(NULL, "Error: Could not create main window", AppName, MB_OK);
    return 0;
  }

  ShowWindow(MainWindow, CmdShow);
  UpdateWindow(MainWindow);


  if (memcmp(CmdLine, "-midi", 5) == 0)
  {
    Playlist.Clear();
    Playlist.Add(CmdLine + 6);
    Playlist.SetCurrentSong(0);
    OpenMIDI(MainWindow, CmdLine + 6);
    PlayMIDI();
  }


  if (memcmp(CmdLine, "-playlist", 9) == 0)
  {
    Playlist.Load(CmdLine + 10);

    if (Playlist.GetNumFiles() != 0)
    {
      if (GetShuffle())
        Playlist.Next();
      else
        Playlist.SetCurrentSong(0);
      OpenMIDI(MainWindow, (char *)Playlist.GetCurrentSong());
    }
  }


  MSG msg;
  while (GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  // Saves config data

  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE", 0, KEY_ALL_ACCESS, &SoftwareKey) == ERROR_SUCCESS)
  {
    HKEY SeraphKey;
    if (RegOpenKeyEx(SoftwareKey, "Seraph", 0, KEY_ALL_ACCESS, &SeraphKey) == ERROR_SUCCESS)
    {
      HKEY MidiPlayKey;
      if (RegOpenKeyEx(SeraphKey, "midiplay", 0, KEY_ALL_ACCESS, &MidiPlayKey) == ERROR_SUCCESS)
      {
        char Data[256];
        GetCurrentDirectory(255, Data);
        RegSetValueEx(MidiPlayKey, "LastDirectory", 0, REG_SZ, (BYTE *)Data, 255);
        itoa(WindowX, Data, 10);
        RegSetValueEx(MidiPlayKey, "WindowX", 0, REG_SZ, (BYTE *)Data, 255);
        itoa(WindowY, Data, 10);
        RegSetValueEx(MidiPlayKey, "WindowY", 0, REG_SZ, (BYTE *)Data, 255);
        RegSetValueEx(MidiPlayKey, "Repeat", 0, REG_SZ, (BYTE *)(GetRepeat() ? "1" : "0"), 255);
        RegSetValueEx(MidiPlayKey, "Shuffle", 0, REG_SZ, (BYTE *)(GetShuffle() ? "1" : "0"), 255);
        RegSetValueEx(MidiPlayKey, "Intro", 0, REG_SZ, (BYTE *)(GetIntro() ? "1" : "0"), 255);
        RegSetValueEx(MidiPlayKey, "PlayOnLoad", 0, REG_SZ, (BYTE *)(PlayOnLoad ? "1" : "0"), 255);

        RegCloseKey(MidiPlayKey);
      }
      else
        MessageBox(NULL, "Error: Could not open registry key", AppName, MB_OK);

      RegCloseKey(SeraphKey);
    }
    else
      MessageBox(NULL, "Error: Could not open registry key", AppName, MB_OK);

    RegCloseKey(SoftwareKey);
  }
  else
    MessageBox(NULL, "Error: Could not open registry key", AppName, MB_OK);

  return msg.wParam;
}