#include <windows.h>
#include <stdio.h>
#include "midiplay.h"
#include "midi.h"


extern long WindowX;
extern long WindowY;


static long Length = 1;
static long CurPos = 0;


void CALLBACK TimerProc(HWND hwnd, UINT msg, UINT event, DWORD time);


RECT RW_Rect = { 10, 50, 300, 90 };

#define RedrawWindow() InvalidateRect(hwnd, &RW_Rect, false)  


// Kewlies struct that helps make and handle buttons
struct XBUTTON
{
  int x, y;
  int bmp_resource;

  HWND hwnd;
  HBITMAP bitmap;

  bool LoadBitmap()
  {
    bitmap = ::LoadBitmap(hInst, MAKEINTRESOURCE(bmp_resource));
    return true;
  }

  bool Create(HWND parent)
  {
    hwnd = CreateWindow("BUTTON", "",
      WS_VISIBLE | BS_PUSHBUTTON | WS_CHILD | BS_OWNERDRAW,
      x, y, 30, 30,
      parent, NULL, hInst, NULL);
    return true;
  }
};


// Handles the text buttons (shuffle, repeat, etc...)
struct TBUTTON
{
  int x, y;
  char text[20];

  HWND hwnd;
  HBITMAP bitmap;

  bool Create(HWND parent)
  {
    hwnd = CreateWindow("BUTTON", text,
      WS_VISIBLE | BS_PUSHBUTTON | WS_CHILD | BS_CENTER,
      x, y, 80, 20,
      parent, NULL, hInst, NULL);
    return true;
  }
};


bool OpenMIDI(HWND hwnd, char *file)
{
  char *fop;  // File Offset Pointer?  hehe
  fop = file + strlen(file);
  while (fop > file && *(fop - 1) != '\\')
    fop--;


  char TempString[300];
  sprintf(TempString, "%s - %s", fop, AppName);
  SetWindowText(hwnd, TempString);

  StopMIDI();
  LoadMIDI(file);

  CurPos = 0;
  Length = GetMIDILength();
  return true;
}


LRESULT CALLBACK MainWindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
  static UINT Timer;

  static int NumButtons = 6;
  static XBUTTON Buttons[] =
  {
    { 10, 10, IDB_PLAY },
    { 40, 10, IDB_PAUSE },
    { 70, 10, IDB_STOP },
    { 120, 10, IDB_PREV },
    { 150, 10, IDB_NEXT },
    { 200, 10, IDB_OPEN },
  };

  static int NumTButtons = 4;
  static TBUTTON TButtons[4] =
  {
    { 310, 10, "Repeat" },
    { 310, 30, "Shuffle" },
    { 310, 50, "Intro" },
    { 310, 70, "Playlist" }
  };

  int i;
  RECT rect;

  switch (msg)
  {
    case WM_CREATE:
      Timer = SetTimer(hwnd, 0, 100, (TIMERPROC)TimerProc);

      for (i = 0; i < NumButtons; i++)
      {
        Buttons[i].Create(hwnd);
        Buttons[i].LoadBitmap();
      }
      for (i = 0; i < NumTButtons; i++)
      {
        TButtons[i].Create(hwnd);
      }

      return 0;

    case WM_DRAWITEM:
      LPDRAWITEMSTRUCT dis;
      dis = (LPDRAWITEMSTRUCT)lparam;

      HDC hdcMemory;
      hdcMemory = CreateCompatibleDC(dis->hDC);

      for (i = 0; i < NumButtons; i++)
        if (dis->hwndItem == Buttons[i].hwnd) SelectObject(hdcMemory, Buttons[i].bitmap);

      // Gets a rect of the button
      rect = dis->rcItem;

      // Fills the rect with light grey
      HBRUSH OldBrush;
      HBRUSH Brush;
      Brush = CreateSolidBrush(RGB(192, 192, 192));
      OldBrush = SelectObject(dis->hDC, Brush);
      Rectangle(dis->hDC, rect.left, rect.top, rect.right, rect.bottom);
      SelectObject(dis->hDC, OldBrush);
      DeleteObject(Brush);

      // Changes the rect dimensions a bit for the LineTo()'s
      rect.right--;
      rect.bottom--;

      HPEN OldPen;
      HPEN Pen;

      if (!(dis->itemState & ODS_SELECTED))
      {
        // Sets the current pen as white
        Pen = CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
        OldPen = SelectObject(dis->hDC, Pen);
      }

      // Draws the top of the button
      MoveToEx(dis->hDC, rect.left, rect.bottom, NULL); // Lower left corner
      LineTo(dis->hDC, rect.left, rect.top);            // Upper left corner
      LineTo(dis->hDC, rect.right, rect.top);           // Upper right corner

      // Restores old pen
      if (!(dis->itemState & ODS_SELECTED))
      {
        SelectObject(dis->hDC, OldPen);
        DeleteObject(Pen);
      }

      if (dis->itemState & ODS_SELECTED)
      {
        // Sets the current pen as white
        Pen = CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
        OldPen = SelectObject(dis->hDC, Pen);
      }

      // Draws the bottom of the button
      MoveToEx(dis->hDC, rect.left, rect.bottom, NULL); // Lower left corner
      LineTo(dis->hDC, rect.right, rect.bottom);        // Lower right corner
      LineTo(dis->hDC, rect.right, rect.top);           // Upper right corner

      if (dis->itemState & ODS_SELECTED)
      {
        SelectObject(dis->hDC, OldPen);
        DeleteObject(Pen);
      }

      // Blits the image on the button
      BitBlt(dis->hDC, rect.left + 5, rect.top + 5, 20, 20,
             hdcMemory, 0, 0, SRCCOPY);

      DeleteDC(hdcMemory);
      return 0;

    case WM_CLOSE:
      if (!IsIconic(hwnd))
      {
        RECT WindowRect;
        GetWindowRect(hwnd, &WindowRect);
        WindowX = WindowRect.left;
        WindowY = WindowRect.top;
      }

      char str[90];
      sprintf(str, "%ld, %ld", WindowX, WindowY);

      KillTimer(hwnd, Timer);

      for (i = 0; i < NumButtons; i++)
        DeleteObject(Buttons[i].bitmap);

      StopMIDI();
      CloseMIDI();
      PostQuitMessage(0);
      return 0;


    case WM_COMMAND:
      switch (LOWORD(wparam))
      {
        case CM_FILEOPEN:
          char filename[256];
          filename[0] = 0;

          OPENFILENAME ofn;
          memset(&ofn, 0, sizeof(ofn));
          ofn.lStructSize = sizeof(ofn);
          ofn.hwndOwner = hwnd;
          ofn.hInstance = hInst;
          ofn.lpstrFilter = "MIDI Files\0*.midi;*.mid\0All Files\0*.*\0";
          ofn.lpstrFile = filename;
          ofn.nMaxFile = 250;
          ofn.lpstrTitle = "Open MIDI File";
          ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
          ofn.lpstrDefExt = "midi";

          if (GetOpenFileName(&ofn) == FALSE) return 0;

          OpenMIDI(hwnd, filename);
          if (PlayOnLoad) PlayMIDI();

          Playlist.Clear();
          Playlist.Add(filename);
          Playlist.SetCurrentSong(0);
          return 0;


        case CM_FILEOPTIONS:
          return DialogBox(hInst, MAKEINTRESOURCE(IDD_OPTIONS), hwnd, (DLGPROC)OptionsDialogProc);

        case CM_FILEEXIT:
          StopMIDI();
          DestroyWindow(hwnd);
          PostQuitMessage(0);
          return 0;

        case CM_README_TXT:
          char WinPath[256];
          GetWindowsDirectory(WinPath, 255);

          char ExecString[256];
          sprintf(ExecString, "%s\\readme.txt", ProgDir);

          ShellExecute(hwnd, "open", "notepad.exe", ExecString, WinPath, SW_SHOWNORMAL);
          return 0;

        case CM_ABOUT:
          return DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, (DLGPROC)AboutDialogProc); 
      }


      // Play
      if (lparam == (short)Buttons[0].hwnd) PlayMIDI();

      // Pause
      if (lparam == (short)Buttons[1].hwnd) PauseMIDI();

      // Stop
      if (lparam == (short)Buttons[2].hwnd) StopMIDI();

      // Prev
      if (lparam == (short)Buttons[3].hwnd)
      {
        if (Playlist.Prev())
        {
          bool Playing = MIDIQueryPlaying();
          OpenMIDI(hwnd, Playlist.GetCurrentSong());
          if (Playing) PlayMIDI();
        }
      }

      // Next
      if (lparam == (short)Buttons[4].hwnd)
      {
        if (Playlist.Next())
        {
          bool Playing = MIDIQueryPlaying();
          OpenMIDI(hwnd, Playlist.GetCurrentSong());
          if (Playing) PlayMIDI();
        }
      }

      // Open
      if (lparam == (short)Buttons[5].hwnd)
      {
        SendMessage(hwnd, WM_COMMAND, CM_FILEOPEN, 0);
      }


      // Repeat
      if (lparam == (short)TButtons[0].hwnd)
      {
        ToggleRepeat();
        RedrawWindow();
      }

      // Shuffle
      if (lparam == (short)TButtons[1].hwnd)
      {
        ToggleShuffle();
        RedrawWindow();
      }

      // Intro
      if (lparam == (short)TButtons[2].hwnd)
      {
        ToggleIntro();
        RedrawWindow();
      }

      // Playlist
      if (lparam == (short)TButtons[3].hwnd)
      {
        int song = PlaylistDialog();
        if (song == -1 && MIDIQueryPlaying() == true)
          return 0;
        if (song == -1) song = 0;
        if (song == -2) return 0;
        Playlist.SetCurrentSong(song);
        if (Playlist.GetNumFiles() > 0)
        {
          OpenMIDI(hwnd, (char *)Playlist.GetValue(song));
          PlayMIDI();
         }
      }

      return 0;


    case WM_LBUTTONDOWN:
      POINT Point;
      Point.x = LOWORD(lparam);
      Point.y = HIWORD(lparam);

      if (Point.x > 10 && Point.x < 300)
        if (Point.y > 50 && Point.y < 90)
        {
          float Position = (float)(Point.x - 10) / (float)290;
          Position *= (float)Length;
          SetMIDIPos(Position);
        }
      return 0;


    case WM_PAINT:
      PAINTSTRUCT ps;
      BeginPaint(hwnd, &ps);

      // Draws background of MIDI area
      RECT BarRect;
      BarRect.left = 10;
      BarRect.top = 50;
      BarRect.right = 300;
      BarRect.bottom = 90;
      Brush = CreateSolidBrush(RGB(100, 0, 200));
      FillRect(ps.hdc, &BarRect, Brush);
      DeleteObject(Brush);

      // Draws foreground of MIDI area
      BarRect.right = ((float)290 * (float)CurPos / (float)Length) + 10;
      Brush = CreateSolidBrush(RGB(200, 0, 100));
      FillRect(ps.hdc, &BarRect, Brush);
      DeleteObject(Brush);

      int BkMode;
      BkMode = SetBkMode(ps.hdc, TRANSPARENT);

      HFONT Font, OldFont;;
      Font = CreateFont(14, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        "Arial");

      OldFont = SelectObject(ps.hdc, Font);

      if (GetRepeat()) SetTextColor(ps.hdc, RGB(255, 255, 255));
      else SetTextColor(ps.hdc, RGB(0, 0, 0));
      TextOut(ps.hdc, 12, 50, "Repeat", 6);

      if (GetShuffle()) SetTextColor(ps.hdc, RGB(255, 255, 255));
      else SetTextColor(ps.hdc, RGB(0, 0, 0));
      TextOut(ps.hdc, 62, 50, "Shuffle", 7);

      if (GetIntro()) SetTextColor(ps.hdc, RGB(255, 255, 255));
      else SetTextColor(ps.hdc, RGB(0, 0, 0));
      TextOut(ps.hdc, 112, 50, "Intro", 5);

      SelectObject(ps.hdc, OldFont);
      DeleteObject(Font);
      SetBkMode(ps.hdc, BkMode);

      // Puts a 3D look around the data rectangle.
      MoveToEx(ps.hdc, 10, 90, NULL);
      LineTo(ps.hdc, 10, 50);
      LineTo(ps.hdc, 300, 50);

      HPEN NewPen;
      NewPen = CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
      OldPen = SelectObject(ps.hdc, NewPen);

      MoveToEx(ps.hdc, 10, 90, NULL);
      LineTo(ps.hdc, 299, 90);
      LineTo(ps.hdc, 299, 50);

      SelectObject(ps.hdc, OldPen);
      DeleteObject(NewPen);
      // Done making it 3D

      EndPaint(hwnd, &ps);
      return 0;

    default: return DefWindowProc(hwnd, msg, wparam, lparam);
  }
}


#pragma argsused
void CALLBACK TimerProc(HWND hwnd, UINT msg, UINT event, DWORD time)
{
  static long OldPos;
  static int  OldIconic;

  if (msg == WM_TIMER)
  {
    if (IsIconic(hwnd) != OldIconic)
    {
      if (!IsIconic(hwnd))
      {
        RECT WindowRect;
        GetWindowRect(hwnd, &WindowRect);
        WindowX = WindowRect.left;
        WindowY = WindowRect.top;
      }
    }
    OldIconic = IsIconic(hwnd);

    if (MIDIQueryPlaying())
    {
      // Does stuff with the MIDI (like restarts if looping, etc)
      CurPos = GetMIDIPos();

      if ((GetIntro() && CurPos > 100) || CurPos == Length)
      {
        StopMIDI(); CurPos = 0;

        if (GetRepeat())
          PlayMIDI();
        else
        {
          int cursong = Playlist.GetCurrentIndex();
          Playlist.Next();
          OpenMIDI(hwnd, Playlist.GetCurrentSong());
          if (Playlist.GetCurrentIndex() != cursong)
            PlayMIDI();
        }
      }

      if (OldPos != CurPos)
        RedrawWindow();
      OldPos = CurPos;
    }
  }
}
